*This do file cleans food consumption data

*Check if filepaths have been established using init.do
if "${gsdData}"=="" {
	display as error "Please run init.do first."
	error 1
}

use "${gsdData}\1-CleanInput\food.dta", clear
*prepare data for cleaning
*merge with sampling weights to calculate medians
merge m:1 state ea hh using "${gsdData}/1-CleanInput/hhq.dta" , assert(match using) keep(match) keepusing(weight hhsize month) nogen
*rename variables of interest 
rename (D_4_cons D_5_cons_quant D_5_cons_unit D_6_purc_quant D_6_purc_unit D_7_pric_total D_7_pric_total_curr) (cons cons_q cons_u purc_q purc_u price price_curr)
*Generate vars that report kg quantities consumed and produced
gen cons_q_kg = cons_q * cons_conv2kg
label var cons_q_kg	"Quantity consumed in kg"
gen purc_q_kg = purc_q * purc_conv2kg
label var purc_q_kg "Quantity purchased in kg"
*gen item id for easily seeing codes in data
gen item=itemid

* Unit correction:
* Check distribution of quantities across units to see if units are systematically misreported or if it is simply outliers
tabstat purc_q , by(purc_u) s(N min p5 p25 p50 mean p75 p95 max)
tabstat cons_q , by(cons_u) s(N min p5 p25 p50 mean p75 p95 max)
*Cleaning rule 1: tag units where a record has the same figure in quantity consumed and purchased but different units
gen tag_same_q_diff_u=1 if (cons_q==purc_q) & (cons_u!=purc_u) & cons==1 & !missing(cons_q) & !missing(purc_q)
replace tag_same_q_diff_u=0 if cons==1 & tag_same_q_diff_u!=1
*The correct unit is the one that takes the variable (consumption or purchase) closer to the weighted 
*  median value in the distribution of that variable for the same item
*calculate the median quantity consumed/purchased
local ls = "cons purc"
foreach s of local ls {
	bysort itemid: egen x`s'_kg_med=median(`s'_q_kg) if tag_same_q_diff_u!=1 & `s'_q_kg!=0 
	bysort itemid: egen `s'_kg_med=max(x`s'_kg_med)
	drop x`s'_kg_med
	*make sure medians are never equal to zero
	assert `s'_kg_med!=0 
}
*obtain the absolute difference between the quantity and the median for the respective item
gen diff_cons_kg=abs(cons_q_kg - cons_kg_med) if tag_same_q_diff_u==1
gen diff_purc_kg=abs(purc_q_kg - purc_kg_med) if tag_same_q_diff_u==1
*introduce the replacements
replace cons_u=purc_u 					if diff_purc_kg<diff_cons_kg & tag_same_q_diff_u==1
replace purc_u=cons_u 					if diff_purc_kg>diff_cons_kg & tag_same_q_diff_u==1
replace cons_conv2kg=purc_conv2kg 		if diff_purc_kg<diff_cons_kg & tag_same_q_diff_u==1
replace purc_conv2kg=cons_conv2kg   	if diff_purc_kg>diff_cons_kg & tag_same_q_diff_u==1
replace cons_q_kg=purc_q_kg 			if diff_purc_kg<diff_cons_kg & tag_same_q_diff_u==1
replace purc_q_kg=cons_q_kg 			if diff_purc_kg>diff_cons_kg & tag_same_q_diff_u==1
* Check if all conversion factors that need to be constant are constant
tabstat purc_conv2kg, by(purc_u) stat(min q max)
tabstat cons_conv2kg, by(cons_u) stat(min q max)
*check impact of cleaning rule
tabstat cons_q , by(cons_u) s(N min p5 p25 p50 mean p75 p95 max)
tabstat purc_q , by(purc_u) s(N min p5 p25 p50 mean p75 p95 max)
*Cleaning rule 2: clean items propably reported in the wrong units
gen tag_cons_wrong_u = 0 if cons==1
gen tag_purc_wrong_u = 0 if cons==1
* Create loop over units of consumption and purchase, tag and then replace
local ls = "cons purc"
foreach s of local ls {
	* Basin (10 litre): If basin (10 litre) is >= 10, divide by 10 because the 
	* enumerator probably meant litres 
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=10 & `s'_u==1 & cons==1 & cons_q!=.)
	replace `s'_q 			= `s'_q/10 		if  (`s'_q>=10 & `s'_u==1 & cons==1 & cons_q!=.)
	* Bundle (100g): If bundle (100g) is >=100, divide by 100 because the 
	* enumerator probably meant grams
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=100 & `s'_u==2 & cons==1  & cons!=.)
	replace `s'_q 			= `s'_q/100 	if  (`s'_q>=100 & `s'_u==2 & cons==1  & cons!=.)
	* Cup (200g): If cup (200g) is >= 100, divide by 200 because the enumerator 
	* probably meant grams
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=100 & `s'_u==3 &  cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/200 	if  (`s'_q>=100 & `s'_u==3 &  cons==1 & cons!=.)
	* Gram: If grams <=5, multiply by 1000 because the enumerator probably meant 
	* kilogram 
	replace tag_`s'_wrong_u = 1 			if  (`s'_q<=5 & `s'_u==5  &  cons==1 & cons!=.)
	replace `s'_q 			= `s'_q*1000	if  (`s'_q<=5 & `s'_u==5  &  cons==1 & cons!=.)
	* Gram: If grams >=10000, divide by 100 because the enumerator probably meant 
	* hundreds of grams 
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=10000 & `s'_u==5  &  cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/100 	if  (`s'_q>=10000 & `s'_u==5  &  cons==1 & cons!=.)
	* Heap (100g): If heap (100g) is >= 100, divide by 100 because the 
	* enumerator probably meant grams
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=100 & `s'_u==6  & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/100 	if  (`s'_q>=100 & `s'_u==6  & cons==1 & cons!=.)
	* Heap (150g): If heap (150g) is >= 150, divide by 150 because the enumerator 
	* probably meant grams
	replace tag_`s'_wrong_u = 1 			if (`s'_q>=150 & `s'_u==7 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/150 	if (`s'_q>=150 & `s'_u==7 & cons==1 & cons!=.)
	* Heap (200g): If heap (200) is >= 200, divide by 200 because the enumerator 
	* probably meant grams
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=200 & `s'_u==8 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/200 	if  (`s'_q>=200 & `s'_u==8 & cons==1 & cons!=.)
	* Heap (300g): If heap (300) is >= 300, divide by 300 because the enumerator 
	* probably meant grams
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=300 & `s'_u==9 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/300 	if  (`s'_q>=300 & `s'_u==9 & cons==1 & cons!=.)
	* Heap (700g): If heap (700g) is >= 100, divide by 700 because the 
	* enumerator probably meant grams
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=100 & `s'_u==10 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/700 	if  (`s'_q>=100 & `s'_u==10 & cons==1 & cons!=.)
	* Jerdal (12kg): If Jerdal is >= 12, divide by 12 because the 
	* enumerator probably meant kilograms
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=12 & `s'_u==11 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/12 	if  (`s'_q>=12 & `s'_u==11 & cons==1 & cons!=.)
	* Kilogram: If kilogram > 100, divide by 1,000 because the enumerator 
	* probably meant grams
	replace tag_`s'_wrong_u = 1 			if (`s'_q>100 & `s'_u==12 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/1000 	if (`s'_q>100 & `s'_u==12 & cons==1 & cons!=.)
	* Litre: If litre >=100, divide by 1,000 because the enumerator probably 
	* meant millilitre i.e. grams
	replace tag_`s'_wrong_u = 1 			if (`s'_q>=100 & `s'_u==13 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/1000 	if (`s'_q>=100 & `s'_u==13 & cons==1 & cons!=.)
	* Millilitre: If millilitres <=5, multiply by 1000 because the enumerator probably meant 
	* Litre 
	replace tag_`s'_wrong_u = 1 			if  (`s'_q<=5 & `s'_u==14  &  cons==1 & cons!=.)
	replace `s'_q 			= `s'_q*1000 	if  (`s'_q<=5 & `s'_u==14  &  cons==1 & cons!=.)
	* Sack (50kg): If sack (50kg) >= 10, divide by 50 because the enumerator 
	* probably meant kilogram
	replace tag_`s'_wrong_u = 1 			if  (`s'_q>=10 & `s'_u==39 & cons==1 & cons!=.)
	replace `s'_q 			= `s'_q/50 		if  (`s'_q>=10 & `s'_u==39 & cons==1 & cons!=.)
}
*check impact of corrections
*Note: Quantities that remain very high at the 99th percentile or maximum will get cleaned during outlier correction in the next step
tabstat cons_q , by(cons_u) s(N min p5 p25 p50 mean p75 p95 p99 max)
tabstat purc_q if purc_q!=0, by(purc_u) s(N min p5 p25 p50 mean p75 p95 p99 max) 
label define ltag 0 "Not-tagged" 1 "Tagged" , replace
label val tag_* ltag
tab tag_same_q_diff_u
qui tabout tag_same_q_diff_u using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , replace c(freq col)
tab tag_cons_wrong_u
qui tabout tag_cons_wrong_u using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
tab tag_purc_wrong_u
qui tabout tag_purc_wrong_u using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
*drop tags and variables used for cleaning this section
drop tag_* *_med diff_*
*Regenerate vars that report kg quantities consumed and produced after corrections
drop cons_q_kg
gen cons_q_kg = cons_q * cons_conv2kg
label var cons_q_kg	"Quantity consumed in kg"
drop purc_q_kg 
gen purc_q_kg = purc_q * purc_conv2kg
label var purc_q_kg "Quantity purchased in kg"
*check whether conversions into kg make sense
tabstat cons_q_kg , by(cons_u) s(N min p5 p25 p50 mean p75 p95 p99 max)
tabstat purc_q_kg if purc_q!=0, by(purc_u) s(N min p5 p25 p50 mean p75 p95 p99 max) 
save "${gsdTemp}/foodcleaningtemp_1.dta", replace


* Cleaning quantities purchased and consumed:
use "${gsdTemp}/foodcleaningtemp_1.dta", clear
*1-tag observations according to cleaning rules
* Cleaning Rule 3: 
* (i) tag values that were consumed but have missing quantities
gen tag_cons_but_no_cons_q=1 if cons==1 & missing(cons_q)
replace tag_cons_but_no_cons_q=0 if cons==1 & tag_cons_but_no_cons_q!=1
* (ii) tag values that were consumed and have missing purchase quantities
gen tag_cons_but_no_purc_q=1 if cons==1  & missing(purc_q)
replace tag_cons_but_no_purc_q=0 if cons==1 & tag_cons_but_no_purc_q!=1
* (iii) tag values that were consumed but have zero consumption quantities
gen tag_cons_but_zero_cons=1 if cons==1 & cons_q==0
replace tag_cons_but_zero_cons=0 if cons==1 & tag_cons_but_zero_cons!=1
* Cleaning Rule 4: 
* Tag values below or above item quantity hard constraints
rename (c_min_h c_max_h p_min_h p_max_h) (cons_min_h cons_max_h purc_min_h purc_max_h)
local ls = "cons purc"
foreach s of local ls{
	gen tag_`s'_below_hard_q = 1 if  (`s'_q<`s'_min_h & cons==1 & !missing(`s'_min_h) & `s'_q!=0 & !missing(`s'_q) )
	replace tag_`s'_below_hard_q=0 if cons==1 & tag_`s'_below_hard_q!=1
	gen tag_`s'_above_hard_q = 1 if  (`s'_q>`s'_max_h & cons==1 & !missing(`s'_max_h) & `s'_q!=0 & !missing(`s'_q) )
	replace tag_`s'_above_hard_q=0 if cons==1 & tag_`s'_above_hard_q!=1 
}
* Cleaning Rule 5: 
* (i) tag values with same figure in quant. consumed, purchased and price
gen tag_same_cons_purc_price=1 if cons==1 & !missing(cons_q) & cons_q == purc_q & purc_q == price 
replace tag_same_cons_purc_price=0 if cons==1 & tag_same_cons_purc_price!=1
* (ii) tag values with same figure for quantity purchased and price 
gen tag_same_purc_price=1 if cons==1 & !missing(purc_q) & purc_q == price & cons_q != purc_q
replace tag_same_purc_price=0 if cons==1 & tag_same_purc_price!=1
* (iii) tag values with same figure for quantity consumed and price 
gen tag_same_cons_price=1 if cons==1 & !missing(cons_q) & cons_q == price & cons_q != purc_q
replace tag_same_cons_price=0 if cons==1 & tag_same_cons_price!=1
* Cleaning Rule 6: 
bys itemid: egen itemcons=total(cons)
local ls = "cons purc"
foreach s of local ls{
	gen `s'_q_kg_pc=`s'_q_kg/hhsize
	bys itemid: egen xmean_`s'=mean(`s'_q_kg_pc) if tag_`s'_above_hard_q==0 
	bys itemid: egen mean_`s'=max(xmean_`s')
	bys itemid: egen xsd_`s'=sd(`s'_q_kg_pc) if tag_`s'_above_hard_q==0 
	bys itemid: egen sd_`s'=max(xsd_`s')
	*replace quantities above 3 standard deviations from the mean for items with more than 300 households consuming it
	gen tag_`s'_above_sd_pc=0 if cons==1
	replace tag_`s'_above_sd_pc=1 if cons==1 & `s'_q_kg_pc>(mean_`s'+(3*sd_`s')) & ! missing(`s'_q_kg_pc) & itemcons>300
}
*tag consumption values that must be replaced according to quantity cleaning rules: 3 (i) and (ii), 4, 5 (i), and 6.
label define ltag 0 "Not-tagged" 1 "Tagged" , replace
gen replace_cons=0 if cons==1
label val replace_cons ltag 
label var replace_cons "Consumption value to be replaced with median" 
local varlist "tag_cons_but_no_cons_q tag_cons_but_zero_cons tag_cons_below_hard_q tag_cons_above_hard_q tag_same_cons_price tag_cons_above_sd_pc" 
foreach t of local varlist {
	replace replace_cons = 1 if (`t'==1)
	label val `t' ltag
	tab `t'
	quietly tabout `t' using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
}
tab replace_cons
qui tabout replace_cons using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col) 
*tag purchase values that must be replaced according to quantity cleaning rules: 3, 5 (i) and (ii), and 6.
gen replace_purc=0 if cons==1
label val replace_purc ltag 
label var replace_purc "Purchase value to be replaced with median"
local varlist "tag_cons_but_no_purc_q tag_purc_below_hard_q tag_purc_above_hard_q tag_same_purc_price tag_purc_above_sd_pc" 
foreach t of local varlist {
	replace replace_purc = 1 if (`t'==1)
	label val `t' ltag
	tab `t'
	quietly tabout `t' using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
}
tab replace_purc
qui tabout replace_purc using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
*2-Calculate medians for quantities consumed and quantities purchased on three levels: (i) EA (ii) strata (state-urban)
*  (iii) national level; count the # of obs available at each level to see how representative the medians
*  are at the respective level
local varlist "cons purc"
foreach v of local varlist {
	bysort itemid state ea: egen x`v'_q_kg_ea_med = median(`v'_q_kg) if replace_`v'==0 & `v'_q_kg!=0
	bysort itemid state ea: egen `v'_q_kg_ea_med = max(x`v'_q_kg_ea_med)
	drop x`v'_q_kg_ea_med
	bysort itemid state ea: egen x`v'_q_kg_ea_c = count(`v'_q_kg) if replace_`v'==0 & `v'_q_kg!=0
	bysort itemid state ea: egen `v'_q_kg_ea_c = max(x`v'_q_kg_ea_c)
	drop x`v'_q_kg_ea_c
	bysort itemid state: egen x`v'_q_kg_state_med = median(`v'_q_kg) if replace_`v'==0 & `v'_q_kg!=0
	bysort itemid state: egen `v'_q_kg_state_med = max(x`v'_q_kg_state_med)
	drop x`v'_q_kg_state_med
	bysort itemid state: egen x`v'_q_kg_state_c = count(`v'_q_kg) if replace_`v'==0 & `v'_q_kg!=0
	bysort itemid state: egen `v'_q_kg_state_c = max(x`v'_q_kg_state_c)
	drop x`v'_q_kg_state_c	
	bysort itemid: egen x`v'_q_kg_itemid_med = median(`v'_q_kg) if replace_`v'==0 & `v'_q_kg!=0
	bysort itemid: egen `v'_q_kg_itemid_med = max(x`v'_q_kg_itemid_med)
	drop x`v'_q_kg_itemid_med
	bysort itemid: egen x`v'_q_kg_itemid_c = count(`v'_q_kg) if replace_`v'==0 & `v'_q_kg!=0
	bysort itemid: egen `v'_q_kg_itemid_c = max(x`v'_q_kg_itemid_c)
	drop x`v'_q_kg_itemid_c		
}
*make sure that medians are not equal to zero
assert cons_q_kg_ea_med!=0
assert purc_q_kg_ea_med!=0
assert cons_q_kg_state_med!=0
assert purc_q_kg_state_med!=0
assert cons_q_kg_itemid_med!=0
assert purc_q_kg_itemid_med!=0
*make sure medians at least at the item level are not missing; check for which items do we have no quantities
tab itemid item if missing(cons_q_kg_itemid_med) & cons==1
assert !missing(cons_q_kg_itemid_med) if cons==1 
*we dont have any purchase quantities for coriander and mineral water
tabstat item if missing(purc_q_kg_itemid_med) & cons==1 , by(itemid) stat(mean N) 
assert !missing(purc_q_kg_itemid_med) if cons==1 & !inlist(itemid,10910,11103)
*inspect medians and counts
tabstat *_med *_c , s(N min p5 q p95 max) columns(stat)
*3-replace tagged outliers with medians
* Use median quantities from smallest geo level where min # of 5/10 obs are available
*check how many items will be affected
tab replace_cons
*perform corrections
replace  cons_q_kg = cons_q_kg_itemid_med if (replace_cons==1) & !missing(cons_q_kg_itemid_med) & cons==1
replace  cons_q_kg = cons_q_kg_state_med if (replace_cons==1 & cons_q_kg_state_c>=10 & !missing(cons_q_kg_state_c)) & cons==1
replace  cons_q_kg = cons_q_kg_ea_med if (replace_cons==1 & cons_q_kg_ea_c>=5 & !missing(cons_q_kg_ea_c)) & cons==1
*check how many items will be affected
tab replace_purc
*perform corrections
replace  purc_q_kg = purc_q_kg_itemid_med if (replace_purc==1) & !missing(purc_q_kg_itemid_med) & cons==1								
replace  purc_q_kg = purc_q_kg_state_med if (replace_purc==1 &  purc_q_kg_state_c>=10 & !missing(purc_q_kg_state_c)) & cons==1
replace  purc_q_kg = purc_q_kg_ea_med if (replace_purc==1 &  purc_q_kg_ea_c>=5 & !missing(purc_q_kg_ea_c)) & cons==1
*manually correct a few items where the consumption quantity was replaced by median quantity consumed which was higher than 10 
* but they are small households and this leads to much too high consumption quantities per capita 
* recalculate per capita consumption first
drop cons_q_kg_pc
gen cons_q_kg_pc=cons_q_kg/hhsize
replace cons_q_kg=cons_q_kg_itemid_med if cons_q_kg_pc>=10 & cons==1
*do the same for quantities purchased 
drop purc_q_kg_pc
gen purc_q_kg_pc=purc_q_kg/hhsize
replace purc_q_kg=purc_q_kg_itemid_med if purc_q_kg_pc>=20 & cons==1
*rename three tags for later use for cleaning prices; change prefix so that they are not dropped in following line
rename (tag_same_cons_purc_price tag_same_purc_price tag_same_cons_price) (keep_tag_same_cons_purc_price keep_tag_same_purc_price keep_tag_same_cons_price)
*drop tags and variables used for cleaning this section
drop tag_* *_med *_c replace_* x* mean_* sd_*
*re-rename tags to keep
rename (keep_tag_same_cons_purc_price keep_tag_same_purc_price keep_tag_same_cons_price) (tag_same_cons_purc_price tag_same_purc_price tag_same_cons_price) 
*4-Check impact of cleaning 
*check that quantity consumed is never equal to zero
assert cons_q_kg!=0
*check impact of corrections
tabstat cons_q_kg , s(N min p5 p25 p50 mean p75 p95 p99 max)
tabstat purc_q_kg , s(N min p5 p25 p50 mean p75 p95 p99 max)
*by itemid and per capita
drop cons_q_kg_pc
drop purc_q_kg_pc
gen cons_q_kg_pc=cons_q_kg/hhsize
gen purc_q_kg_pc=purc_q_kg/hhsize
tabstat cons_q_kg cons_q_kg_pc , by(itemid) s(N min p5 p25 p50 mean p75 p95 p99 max) format(%8.2g) col(stat)
tabstat purc_q_kg purc_q_kg_pc , by(itemid) s(N min p5 p25 p50 mean p75 p95 p99 max) format(%8.2g) col(stat)
*relabel quantities
label var cons_q_kg	"Quantity consumed in kg, after cleaning"
label var purc_q_kg "Quantity purchased in kg, after cleaning"
save "${gsdTemp}/foodcleaningtemp_2.dta", replace


* Cleaning: Prices 
use "${gsdTemp}/foodcleaningtemp_2.dta", clear 
*1-Prepare prices
*Convert currencies
list itemid item purc_q purc_u price price_curr D_7_pric_total_sp if price_curr==3
replace price=. if D_7_pric_total_sp=="." & price_curr==3
gen curr="UGX" if D_7_pric_total_sp==`""Ugandan shillings ""'
*two prices are given with "other" currency specified as "Dura", replace as missing since prices are too low to be real
replace price=. if inlist(itemid,10714,10303) & D_7_pric_total_sp==`""dura ""'
gen year=2017
merge m:1 curr year using "${gsdData}/1-CleanInput/Other/exchange_rates_year.dta", nogen assert(match master using) keep(match master) 
gen price_ssp=price*(conv2ssp/conv2usd) if curr=="UGX"
*inspect prices in USD // there are 9 prices which don't make sense if converted into USD
list itemid purc_q purc_u price price_curr D_7_pric_total_sp if price_curr==2 
replace price_curr=1 if price_curr==2
drop conv2* curr year
*generate price variable
replace price_ssp=price if price_curr==1
label var price_ssp "Price paid in current SSP"
*get CPI to calculate hard price constraints, since they were removed during fieldwork due to the increasing inflation
merge m:1 month using "${gsdData}/1-CleanInput/Other/CPI_NBS.dta", nogen assert(match using) keep(match) keepusing(CPI_food ref_CPI_food_Feb2015)
gen scale_factor=CPI_food/ref_CPI_food_Feb2015
gen pr_max=v_max*scale_factor
* Calculate per kilo price
gen  price_ssp_per_kg = price_ssp/purc_q_kg if cons==1
label var price_ssp_per_kg "Price paid per kg in current SSP"
*2-Tag observations according to cleaning rules
*Cleaning rule 6: replace missing prices
* (i) tag values that were purchased but have missing prices; 
gen tag_purc_but_no_price=1 if cons==1 & missing(price) & !missing(purc_q) & purc_q!=0
replace tag_purc_but_no_price=0 if cons==1 & tag_purc_but_no_price!=1
* (ii) tag items where quantity purchased is zero or missing to replace the missing prices
gen tag_purc_zero_missing = 1 if cons==1 &  (purc_q_kg==0 | purc_q_kg==.) 
replace tag_purc_zero_missing=0 if cons==1 & tag_purc_zero_missing!=1
* (iii) tag items that were consumed but do not have a price
gen tag_cons_but_no_price = 0 if cons==1
replace tag_cons_but_no_price = 1 if cons==1 & missing(price)
* (iv) tag items that were consumed but have zero price
gen tag_cons_zero_price = 0 if cons==1
replace tag_cons_zero_price = 1 if cons==1 & price==0
* Cleaning Rule 7: 
* Tag bottom and top percent prices in overall price distributions 
local bottom = 0.05
local top = 0.95
* Calculate distribution
cumul  price_ssp_per_kg, gen(price_ssp_per_kg_distribution) equal 
* (i) tag prices at the bottom of the distribution
gen tag_price_bottom=0 if (price_ssp_per_kg_distribution>=`bottom' ) & cons==1 & !missing(price_ssp_per_kg)
replace tag_price_bottom=1 if (price_ssp_per_kg_distribution<`bottom') & cons==1 & !missing(price_ssp_per_kg)
* (ii) tag prices at the top of the distribution
gen tag_price_top=0	if (price_ssp_per_kg_distribution<=`top' ) & cons==1 & !missing(price_ssp_per_kg)
replace tag_price_top=1	if (price_ssp_per_kg_distribution>`top') & cons==1 & !missing(price_ssp_per_kg)
*cleaning rule 8:
* tag prices above hard constraints scaled up for inflation
gen tag_price_above_hard_q = 0 if cons==1 
replace tag_price_above_hard_q = 1 if (price_ssp_per_kg>pr_max) & cons==1 & !missing(price_ssp_per_kg)
* Cleaning Rule 9: 
*replace prices above 3 standard deviations from the mean for items with more than 300 households consuming it
bys itemid: egen xmean_price=mean(price_ssp_per_kg) if tag_price_top==0 
bys itemid: egen mean_price=max(xmean_price)
bys itemid: egen xsd_price=sd(price_ssp_per_kg) if tag_price_top==0 
bys itemid: egen sd_price=max(xsd_price)
gen tag_price_above_sd=0 if cons==1
replace tag_price_above_sd=1 if cons==1 & price_ssp_per_kg>(mean_price+(3*sd_price)) & ! missing(price_ssp_per_kg) & itemcons>200
*check how many values are affected by cleaning rules
label define ltag 0 "Not-tagged" 1 "Tagged" , replace
*tag purchase values that must be replaced according to quantity cleaning rules: 6 (i) and (ii), and 7 (ii).
gen replace_price=0 if cons==1
label var replace_price "Price value to be replaced with median"
local varlist "tag_purc_but_no_price tag_purc_zero_missing tag_cons_but_no_price tag_cons_zero_price tag_same_cons_purc_price tag_same_purc_price tag_same_cons_price tag_price_top tag_price_above_hard_q tag_price_above_sd"
foreach t of local varlist {
	replace replace_price = 1 if (`t'==1)
	label val `t' ltag
	tab `t' 
	quietly tabout `t' using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
}
label val replace_price ltag
tab replace_price 
qui tabout replace_price using "${gsdOutput}/Cleaning/cleaning_rules-food.xls" , append c(freq col)
*3-Calculate medians for per kilo prices on three levels: (i) EA (ii) strata, i.e. 
* state level, (iii) global level; count # of obs available at each level 
* to see how representative the medians are at the respective level
*calculate medians and counts
bysort itemid state ea: egen xprice_ssp_per_kg_ea_med = median(price_ssp_per_kg) if replace_price==0 & price_ssp_per_kg!=0
bysort itemid state ea: egen price_ssp_per_kg_ea_med = max(xprice_ssp_per_kg_ea_med)
drop xprice_ssp_per_kg_ea_med
bysort itemid state ea: egen xprice_ssp_per_kg_ea_c = count(price_ssp_per_kg) if replace_price==0 & price_ssp_per_kg!=0
bysort itemid state ea: egen price_ssp_per_kg_ea_c = max(xprice_ssp_per_kg_ea_c)
drop xprice_ssp_per_kg_ea_c
bysort itemid state: egen xprice_ssp_per_kg_state_med = median(price_ssp_per_kg) if replace_price==0 & price_ssp_per_kg!=0
bysort itemid state: egen price_ssp_per_kg_state_med = max(xprice_ssp_per_kg_state_med)
drop xprice_ssp_per_kg_state_med
bysort itemid state: egen xprice_ssp_per_kg_state_c = count(price_ssp_per_kg) if replace_price==0 & price_ssp_per_kg!=0
bysort itemid state: egen price_ssp_per_kg_state_c = max(xprice_ssp_per_kg_state_c)
drop xprice_ssp_per_kg_state_c
bysort itemid: egen xprice_ssp_per_kg_itemid_med = median(price_ssp_per_kg) if replace_price==0 & price_ssp_per_kg!=0
bysort itemid: egen price_ssp_per_kg_itemid_med = max(xprice_ssp_per_kg_itemid_med)
drop xprice_ssp_per_kg_itemid_med
bysort itemid: egen xprice_ssp_per_kg_itemid_c = count(price_ssp_per_kg) if replace_price==0 & price_ssp_per_kg!=0
bysort itemid: egen price_ssp_per_kg_itemid_c = max(xprice_ssp_per_kg_itemid_c)
drop xprice_ssp_per_kg_itemid_c
*make sure that medians at the itemlevel are never equal to zero
assert price_ssp_per_kg_itemid_med!=0
*There are a few items for which even medians at itemlevel are missing because all prices were tagged to be replaced, 
* manual inspection of prices reveal that they are not out of order so we recalculate the median including tagged prices 
tabstat item  if missing(price_ssp_per_kg_itemid_med) & cons==1, by(itemid) stat(mean N)
local itemlist = "10118 10711 10807 11105 11404 10404 10606 10903 10904 11110 11306 11403 10805" 
foreach i of local itemlist {
	quietly su price_ssp_per_kg [aw=weight] if itemid==`i' & price_ssp_per_kg!=0 , detail
	replace price_ssp_per_kg_itemid_med = r(p50) if itemid==`i' & price_ssp_per_kg_itemid_med!=0 & cons==1
}
*there are many households (18) consuming tinned pulses, replace with the overall median price; also replace for coriander and mineral water (only 4 instances so impact is minimal)
su price_ssp_per_kg [aw=weight] if price_ssp_per_kg!=0 , detail
replace price_ssp_per_kg_itemid_med = r(p50) if inlist(itemid,10723,10910,11103,11006,10725) & price_ssp_per_kg_itemid_med!=0 & cons==1
assert !missing(price_ssp_per_kg_itemid_med) if cons==1 
*inspect medians and counts
tabstat *_med *_c , s(N min p5 q p95 max)
*4-Replace tagged outliers with medians
gen price_ssp_per_kg_cleaned = price_ssp_per_kg if cons==1
label var price_ssp_per_kg_cleaned "Price paid per kg in current SSP after cleaning"
*check how many items will be affected
tab replace_price
*perform corrections
replace  price_ssp_per_kg_cleaned =  price_ssp_per_kg_itemid_med if cons==1 & replace_price==1 & !missing(price_ssp_per_kg_itemid_med)
replace  price_ssp_per_kg_cleaned =  price_ssp_per_kg_state_med if cons==1 & replace_price==1 &  price_ssp_per_kg_state_c>=10 & !missing(price_ssp_per_kg_state_c)
replace  price_ssp_per_kg_cleaned =  price_ssp_per_kg_ea_med if cons==1 & replace_price==1 &  price_ssp_per_kg_ea_c>=5 & !missing(price_ssp_per_kg_ea_c)
*5-Check impact of cleaning
tabstat price_ssp_per_kg price_ssp_per_kg_cleaned , stat(N sd min p5 q p95 p99 max) col(stat)
tabstat price_ssp_per_kg_cleaned , by(itemid) stat(N sd min p5 q p95 p99 max) col(stat)
*make sure prices are never zero 
assert price_ssp_per_kg_cleaned!=0
*make sure prices are never missing for items that are consumed
assert !missing(price_ssp_per_kg_cleaned) if cons==1 
*drop variables used for cleaning
drop *_med *_c tag_* replace_* *_distribution *q_kg_pc month CPI* ref_CPI* scale_factor x* mean_* sd_*
save "${gsdTemp}/foodcleaningtemp_3.dta", replace


*Clean up and save
use "${gsdTemp}/foodcleaningtemp_3.dta", clear
*Fill in zeros for non-consumed items
*Merge mod_hh variable
merge m:1 state ea hh using "${gsdData}/1-CleanInput/hhq.dta", keepusing(mod_hh) keep(match master) assert(match master) nogen
*calculate SSP kg consumption and purchase values for each observation by multiplying the consumed kg quantity with the price per kg
gen cons_value = .
replace cons_value = cons_q_kg*price_ssp_per_kg_cleaned 
label var cons_value "Weekly consumption value of {product} based on quantity consumed and price paid in current SSP after cleaning"
*make sure consumption value is not missing for items that are consumed
assert !missing(cons_value) if cons==1 
*differentiate between missing values and zeros
rename mod_opt mod_item 
replace cons_value = 0 if missing(cons_value) & (mod_hh ==  mod_item)
*fill in zeros for asked but non-consumed core items
replace cons_value = 0 if missing(cons_value) & mod_item == 0
*set consumption value to missing where cons is missing
replace cons_value = . if missing(cons)
*Drop households without core consumption
bys state ea hh: egen xcons_hh0 = sum(cons_value) if mod_item == 0
bys state ea hh: egen cons_hh0 = max(xcons_hh0)
drop if cons_hh0==0 | missing(cons_hh0)
*drop redundant variables
drop xcons_hh0 cons_hh0 mod_hh hhsize weight item itemcons pr_max
*rename
rename ( cons_q cons_u purc_q purc_u price price_curr) ( D_5_cons_quant D_5_cons_unit D_6_purc_quant D_6_purc_unit D_7_pric_total D_7_pric_total_curr)
*reorder
order mod_item , after(itemid)
order  price_ssp price_ssp_per_kg price_ssp_per_kg_cleaned cons_value , after(D_7_pric_total_curr)
order cons_q_kg , after(D_5_cons_unit)
order purc_q_kg , after(D_6_purc_unit)
save "${gsdData}/1-CleanTemp/food.dta" , replace


* Calculate hh level aggregates 
*Calculate weekly hh consumption aggregates for each module
forval i=0/4 {
	*Load food.dta data
	use "${gsdData}\1-CleanTemp\food.dta", clear
	*Merge mod_hh variable
	merge m:1 state ea hh using "${gsdData}/1-CleanInput/hhq.dta", keepusing(mod_hh) keep(match master) assert(match master) nogen	
	*Collapse on hh level for modules 0-4
	collapse mod_item mod_hh (sum) cons_value if mod_item == `i' , by(state ea hh)
	rename cons_value food_opt`i'_hh_wk
	label var food_opt`i'_hh_wk "Total weekly consumption of module `i' in current SSP"
	save "${gsdTemp}/food_opt`i'_hh_wk.dta", replace
}
* Merge into one dataset 
use "${gsdTemp}/food_opt0_hh_wk.dta", clear
forval i=1/4 {
	merge 1:1 state ea hh using "${gsdTemp}/food_opt`i'_hh_wk.dta", keepusing(food_opt`i'_hh_wk) keep(match master) assert(match master) nogen
}	
*Replace missing values with zeros where no items were consumed, but do not assume that households did not consume any food at all
forval i=1/4 {
	replace food_opt`i'_hh_wk = 0 if food_opt`i'_hh_wk  == . & mod_hh == `i'
	replace food_opt`i'_hh_wk = . if food_opt0_hh_wk == 0 | missing(food_opt0_hh_wk)
	replace food_opt`i'_hh_wk = . if mod_hh!=`i'
}
*Check for consistency
tabstat food_opt?_hh_wk, stat(n mean)
*Drop redundant variables
drop mod_hh 
save "${gsdData}/1-CleanTemp/hh_food.dta", replace 


* Create tables to report the impact of cleaning
use "${gsdData}/1-CleanTemp/food.dta" , clear
merge m:1 state ea hh using "${gsdData}/1-CleanInput/hhq.dta" , assert(match using) keepusing(hhsize) nogen
merge m:1 state ea hh using "${gsdData}/1-CleanTemp/hh_food.dta" , assert(match master) keepusing(food_opt0_hh_wk) nogen
replace cons_value = cons_value/hhsize 
gen cons_q_kg_pc=cons_q_kg/hhsize
gen purc_q_kg_pc=purc_q_kg/hhsize
replace cons_value=. if cons_value==0
label var price_ssp_per_kg_cleaned "Price kg clean"
label var cons_value "HH consumption pc"
label var cons_q_kg_pc "cons q cleaned pc"
label var purc_q_kg_pc "purc q cleaned pc"
gen item=itemid
keep dataset state itemid item price_ssp_per_kg_cleaned cons_value cons_q_kg_pc purc_q_kg_pc hhsize food_opt0_hh_wk
save "${gsdTemp}/food_checking_tables.dta", replace
local va  "price_ssp_per_kg_cleaned cons_value cons_q_kg_pc purc_q_kg_pc" 
*Overall
foreach v of local va {
	use  "${gsdTemp}/food_checking_tables.dta" , clear
	local name : variable label `v'
	collapse (count) N = `v' (mean) mean = `v' (min) min = `v' (p1) p1 = `v' (p5) p5 = `v' (p25) p25 = `v' (p50) median = `v' (p75) p75 = `v' (p95) p95 = `v' (max) max = `v' , by(itemid item)
	gsort -N 
	export excel using "${gsdOutput}/Cleaning/Check food cleaning.xlsx", first(var) sheet("Overall `name'") sheetreplace
}
*by state
use  "${gsdTemp}/food_checking_tables.dta", clear
levelsof dataset, local(data)
foreach s of local data {
	foreach v of local va {
		use "${gsdTemp}/food_checking_tables.dta" , clear
		local name : variable label `v'
		keep if state==`s'
		collapse (count) N = `v' (mean) mean = `v' (min) min = `v' (p1) p1 = `v' (p5) p5 = `v' (p25) p25 = `v' (p50) median = `v' (p75) p75 = `v' (p95) p95 = `v' (max) max = `v' , by(itemid item)
		gsort -N 
		export excel using "${gsdOutput}/Cleaning/Check food cleaning.xlsx", first(var) sheet("`s' `name'") sheetreplace
	}
}

