*this do file cleans nonfood consumption data

*Check if filepaths have been established using init.do
if "${gsdData}"=="" {
	display as error "Please run init.do first."
	error 1
}
set more off, perm

use "${gsdData}\1-CleanInput\nonfood.dta", clear
*Add variable on household module assignment and weights to calculate medians
merge m:1 state ea hh using "${gsdData}\1-CleanInput\hhq.dta", keepusing(mod_hh weight month hhsize) assert(match master) keep(match master) nogen
*rename variables for convenience
rename (E_2_cons E_3_pric_total E_3_pric_total_curr) (cons price curr)
gen item=itemid
sort curr
list item itemid price curr E_3_pric_total_sp if inlist(curr,2,3)
*exchange prices not in SSP
merge m:1 month using "${gsdData}/1-CleanInput/Other/exchange_rates_month.dta", nogen assert(match master using) keep(master match) keepusing(usdssp usdsdd)
*first make some manual corrections for prices that are indicated as being in ssp or free with a price of zero
replace curr=1 if inlist(itemid,20608,20202,20608,20403,20402,20204,20203,20402,20601,20602) & curr==3
*replace one value of price as zero because the price is for something else, 
list item itemid price curr E_3_pric_total_sp if inlist(curr,3) & itemid==21901
replace curr=1 if curr==3 & itemid==21901 
*make manual corrections for prices that dont make sense once converted
replace curr=1 if itemid==20203 & curr==3
replace curr=1 if itemid==20601 & curr==2
replace curr=1 if itemid==20602 & curr==3
replace curr=1 if itemid==20604 & curr==3
gen price_ssp=price if curr==1 
label var price_ssp "Price in SSP, current"
replace price_ssp=price*usdssp if curr==2
replace price_ssp=price*(usdsdd/usdssp) if curr==3 & inlist(itemid,20501)
list item itemid price price_ssp curr E_3_pric_total_sp if inlist(curr,2)
list item itemid price price_ssp curr E_3_pric_total_sp if inlist(curr,3)
*check if conversion are correct
levelsof itemid if inlist(curr,2,3), local(items)
foreach i of local items {
	tab itemid if itemid==`i'
	su price price_ssp if itemid==`i' & curr==1 & price!=0
	su price price_ssp if itemid==`i' & inlist(curr,2,3) & price!=0
}
*scale constraints up by inflation, because they were created for Wave 1 prices
merge m:1 month using "${gsdData}/1-CleanInput/Other/CPI_NBS.dta", nogen assert(match using ) keep(match)
gen scale_factor=CPI_nonfood/ref_CPI_nonfood_Feb2015
*only replace high constraints because low constraints are more useful to catch outliers, also lower consumption across the board is expected due to conditions in SSD
replace v_max_h=v_max_h*scale_factor
drop scale_factor

*Cleaning: Tagging of outliers
* Cleaning Rules: 
* i) Tag items higher than hard constraint
gen tag_beyond_hard_max= 0 if cons==1
replace tag_beyond_hard_max  = 1 if cons==1 & price_ssp>v_max_h  & !missing(price_ssp)
* ii) Tag items lower than 10 SSP or between 100 and v_min (we do not use just the hard constraint because most items are monthly water fees or traditional healers, hence likely ot have remained cheap)
gen tag_beyond_hard_min= 0 if cons==1 
replace tag_beyond_hard_min= 1 if cons==1 & price_ssp<v_min_h & !missing(price_ssp)
* iii) Tag items with price == 0 
gen tag_zero_price = 0 if cons==1
replace tag_zero_price = 1 if cons==1 & price_ssp == 0
* iv) Tag items that were consumed but have missing price
gen tag_missing_price = 0 if cons==1
replace tag_missing_price = 1 if cons==1 & missing(price_ssp)
*check how manny observations are affected by each cleaning rule
local varlist "tag_beyond_hard_max tag_beyond_hard_min tag_zero_price tag_missing_price"
gen replace_price=0 if cons==1
foreach t of local varlist {
	replace replace_price = 1 if (`t'==1)
}
label var replace_price "Observation to be replaced with median"
label define ltag 0 "Not-tagged" 1 "Tagged"
label val replace_price ltag
quietly tabout replace_price using "${gsdOutput}/Cleaning/cleaning_rules-nonfood.xls" , replace c(freq col)
ds tag_*
foreach v in `r(varlist)' {
	tab `v'
	label val `v' ltag
	quietly tabout `v' using "${gsdOutput}/Cleaning/cleaning_rules-nonfood.xls" , append c(freq col)
}
*Calculate weighted medians at various geographical levels, 
bysort itemid state ea: egen xprice_ea_med = median(price_ssp) if replace_price==0 & price_ssp!=0
bysort itemid state ea: egen price_ea_med = max(xprice_ea_med)
drop xprice_ea_med
bysort itemid state ea: egen xprice_ea_c = count(price_ssp) if replace_price==0 & price_ssp!=0
bysort itemid state ea: egen price_ea_c = max(xprice_ea_c)
drop xprice_ea_c
bysort itemid state: egen xprice_state_med = median(price_ssp) if replace_price==0 & price_ssp!=0
bysort itemid state: egen price_state_med = max(xprice_state_med)
drop xprice_state_med
bysort itemid state: egen xprice_state_c = count(price_ssp) if replace_price==0 & price_ssp!=0
bysort itemid state: egen price_state_c = max(xprice_state_c)
drop xprice_state_c
bysort itemid: egen xprice_itemid_med = median(price_ssp) if replace_price==0 & price_ssp!=0
bysort itemid: egen price_itemid_med = max(xprice_itemid_med)
drop xprice_itemid_med
bysort itemid: egen xprice_itemid_c = count(price_ssp) if replace_price==0 & price_ssp!=0
bysort itemid: egen price_itemid_c = max(xprice_itemid_c)
drop xprice_itemid_c
*make sure that the median prices are never equal to zero
assert price_itemid_med!=0
*inspect medians and counts
tabstat *_med *_c , s(N min p5 q p95 max)
tab itemid item if missing(price_itemid_med) &  cons==1
assert !missing(price_itemid_med) if !inlist(itemid,20901,21805,21804,20704) & cons==1
*2-Replace outliers with median prices as calculated above
*check how many items will be affected
tab replace_price
* Use median prices from smallest geo level where min # of 10 obs are available
gen price_ssp_cleaned = price_ssp if (cons==1)
*replace prices above maximum constraint
replace  price_ssp_cleaned =  price_itemid_med if (replace_price==1)  										
replace  price_ssp_cleaned =  price_state_med if (replace_price==1 &  price_state_c>=10 & price_state_med!=0 & !missing(price_state_med))
replace  price_ssp_cleaned =  price_ea_med if (replace_price==1 &  price_ea_c>=6 & price_ea_med!=0 & !missing(price_ea_med))
*3-Check effect of cleaning on price variables:
tabstat price_ssp price_ssp_cleaned, by(itemid) stat(N mean min p1 p5 q p95 p99 max) col(stat)
*assert prices are not missing if tagged
assert !missing(price_ssp_cleaned) if replace_price==1 & cons==1 & !inlist(itemid,20901,21805,21804,20704)
*make sure that prices are never zero
assert price_ssp_cleaned!=0
su price_ssp_cleaned
label var price_ssp_cleaned "Price of item in current SSP after cleaning"
*Transform annual consumptions into monthly consumptions
gen price_ssp_monthly_cleaned = price_ssp_cleaned if cons==1
replace price_ssp_monthly_cleaned = price_ssp_cleaned/12 if item_recall==365 & cons==1
*Transform monthly consumptions into weekly consumptions
gen cons_value = price_ssp_monthly_cleaned*12/365.25*7 if cons==1 
label var cons_value "Weekly consumption value in current SSP"
*drop variables used for cleaning in this section
drop price_ssp *_ea* *_state* *_itemid* *_monthly* tag_* replace_* CPI* ref_* usd* 

*Clean up and save
*drop households without any core food consumption
merge m:1 state ea hh using "${gsdData}/1-CleanTemp/hh_food.dta", nogen assert(match master) keep(match) keepusing(food_opt0_hh_wk)
drop if food_opt0_hh_wk==0
drop food_opt0_hh_wk
*Replace missing values with zeros where no items were consumed, 
rename mod_opt mod_item
replace cons_value = 0 if (missing(cons_value) & mod_item == 0 ) | ( missing(cons_value) & mod_item==mod_hh)
*Set cons_value to missing where cons is missing, but make sure that it results in no loss of information
assert missing(price) & missing(price_ssp_cleaned)  if missing(cons)
replace cons_value = . if missing(cons)
*make sure that consumption value is missing for the modules not asked to households
assert missing(cons_value) if mod_hh!=mod_item & mod_item!=0
*rename
rename  ( price curr) ( E_3_pric_total E_3_pric_total_curr) 
*drop redundant variables
drop mod_hh weight month hhsize item
*reorder and save
order price_ssp_cleaned, after(E_3_pric_total)
order cons_value, after(cons)
*relabel constraints
label var v_min_s "Minimum soft constraint, adjusted for inflation and used in questionnaire"
label var v_min_h "Minimum hard constraint, adjusted for inflation and used in cleaning"
label var v_max_s "Maximum soft constraint, adjusted for inflation and used in questionnaire"
label var v_max_h "Maximum hard constraint, adjusted for inflation and used in cleaning"
save "${gsdData}/1-CleanTemp/nonfood.dta", replace 

* Calculate hh aggregates 
*Calculate weekly hh consumption aggregates for each module
forval i=0/4 {
	*Load nonfood.dta data
	use "${gsdData}/1-CleanTemp/nonfood.dta", clear
	*get mod_hh variable
	merge m:1 state ea hh using "${gsdData}/1-CleanInput/hhq.dta",  assert(match using) keep(match master) keepusing(mod_hh) nogen
	*Collapse on hh level for modules 0-4
	collapse mod_item mod_hh (sum) cons_value if mod_item == `i' , by(state ea hh)
	rename cons_value nonfood_opt`i'_hh_wk
	save "${gsdTemp}/nonfood_opt`i'_hh_wk.dta", replace
}
*Merge into one dataset 
use "${gsdTemp}/nonfood_opt0_hh_wk.dta", clear
forval i=1/4 {
	merge 1:1 state ea hh  using "${gsdTemp}/nonfood_opt`i'_hh_wk.dta", keepusing(nonfood_opt`i'_hh_wk) keep(match master) assert(match master) nogen
}	
*Replace missing values with zeros where no items were consumed
forvalues i=1/4 {
	replace nonfood_opt`i'_hh_wk = 0 if nonfood_opt`i'_hh_wk  == . & mod_hh == `i'
}
*Set to missing any non-food cons where FOOD core cons was == 0 or . 
merge m:1 state ea hh using  "${gsdData}/1-CleanTemp/hh_food.dta", keepusing(food_opt0_hh_wk) assert(match master) keep(match master)
forvalues i = 1/4 {
	replace nonfood_opt`i'_hh_wk = . if (food_opt0==0 | missing(food_opt0))
	replace nonfood_opt`i'_hh_wk = . if mod_hh!=`i'
}
*Check for consistency
tabstat nonfood_opt?_hh_wk, stat(n mean)
*drop redundant variables	
drop _merge food_opt0_hh_wk mod_hh 
save "${gsdData}/1-CleanTemp/hh_nonfood.dta", replace 

* Create tables to report the impact of cleaning
use "${gsdData}/1-CleanTemp/nonfood.dta" , clear
merge m:1 state ea hh using "${gsdData}/1-CleanInput/hhq.dta" , assert(match using) keepusing(hhsize) keep(match) nogen
replace cons_value = cons_value/hhsize 
replace cons_value=. if cons_value==0
label var price_ssp_cleaned "Price clean"
label var cons_value "HH consumption pc"
save "${gsdTemp}/nonfood_checking_tables.dta", replace
local va  "price_ssp_cleaned cons_value" 
*Overall
foreach v of local va {
	use  "${gsdTemp}/nonfood_checking_tables.dta" , clear
	local name : variable label `v'
	collapse (count) N = `v' (mean) mean = `v' (min) min = `v' (p1) p1 = `v' (p5) p5 = `v' (p25) p25 = `v' (p50) median = `v' (p75) p75 = `v' (p95) p95 = `v' (max) max = `v' , by(itemid)
	gsort -N 
	export excel using "${gsdOutput}/Cleaning/Check nonfood cleaning.xlsx", first(var) sheet("Overall `name'") sheetreplace
}
*by state
use  "${gsdTemp}/nonfood_checking_tables.dta", clear
levelsof dataset, local(dataset)
foreach s of local dataset {
	foreach v of local va {
		use "${gsdTemp}/nonfood_checking_tables.dta" , clear
		local name : variable label `v'
		keep if dataset==`s'
		collapse (count) N = `v' (mean) mean = `v' (min) min = `v' (p1) p1 = `v' (p5) p5 = `v' (p25) p25 = `v' (p50) median = `v' (p75) p75 = `v' (p95) p95 = `v' (max) max = `v' , by(itemid)
		gsort -N 
		export excel using "${gsdOutput}/Cleaning/Check nonfood cleaning.xlsx", first(var) sheet("`s' `name'") sheetreplace
	}
}
